# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class MessagingConfigurationList(ListResource):

    def __init__(self, version, service_sid):
        """
        Initialize the MessagingConfigurationList

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service that the resource is associated with

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationList
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationList
        """
        super(MessagingConfigurationList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, }
        self._uri = '/Services/{service_sid}/MessagingConfigurations'.format(**self._solution)

    def create(self, country, messaging_service_sid):
        """
        Create the MessagingConfigurationInstance

        :param unicode country: The ISO-3166-1 country code of the country or `all`.
        :param unicode messaging_service_sid: The SID of the Messaging Service used for this configuration.

        :returns: The created MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        data = values.of({'Country': country, 'MessagingServiceSid': messaging_service_sid, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return MessagingConfigurationInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
        )

    def stream(self, limit=None, page_size=None):
        """
        Streams MessagingConfigurationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists MessagingConfigurationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of MessagingConfigurationInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return MessagingConfigurationPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of MessagingConfigurationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return MessagingConfigurationPage(self._version, response, self._solution)

    def get(self, country):
        """
        Constructs a MessagingConfigurationContext

        :param country: The ISO-3166-1 country code of the country or `all`.

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        """
        return MessagingConfigurationContext(
            self._version,
            service_sid=self._solution['service_sid'],
            country=country,
        )

    def __call__(self, country):
        """
        Constructs a MessagingConfigurationContext

        :param country: The ISO-3166-1 country code of the country or `all`.

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        """
        return MessagingConfigurationContext(
            self._version,
            service_sid=self._solution['service_sid'],
            country=country,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.MessagingConfigurationList>'


class MessagingConfigurationPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the MessagingConfigurationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: The SID of the Service that the resource is associated with

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationPage
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationPage
        """
        super(MessagingConfigurationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of MessagingConfigurationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        return MessagingConfigurationInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.MessagingConfigurationPage>'


class MessagingConfigurationContext(InstanceContext):

    def __init__(self, version, service_sid, country):
        """
        Initialize the MessagingConfigurationContext

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service that the resource is associated with
        :param country: The ISO-3166-1 country code of the country or `all`.

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        """
        super(MessagingConfigurationContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'country': country, }
        self._uri = '/Services/{service_sid}/MessagingConfigurations/{country}'.format(**self._solution)

    def update(self, messaging_service_sid):
        """
        Update the MessagingConfigurationInstance

        :param unicode messaging_service_sid: The SID of the Messaging Service used for this configuration.

        :returns: The updated MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        data = values.of({'MessagingServiceSid': messaging_service_sid, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return MessagingConfigurationInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            country=self._solution['country'],
        )

    def fetch(self):
        """
        Fetch the MessagingConfigurationInstance

        :returns: The fetched MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return MessagingConfigurationInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            country=self._solution['country'],
        )

    def delete(self):
        """
        Deletes the MessagingConfigurationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.MessagingConfigurationContext {}>'.format(context)


class MessagingConfigurationInstance(InstanceResource):

    def __init__(self, version, payload, service_sid, country=None):
        """
        Initialize the MessagingConfigurationInstance

        :returns: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        super(MessagingConfigurationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'country': payload.get('country'),
            'messaging_service_sid': payload.get('messaging_service_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'service_sid': service_sid, 'country': country or self._properties['country'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: MessagingConfigurationContext for this MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationContext
        """
        if self._context is None:
            self._context = MessagingConfigurationContext(
                self._version,
                service_sid=self._solution['service_sid'],
                country=self._solution['country'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The SID of the Service that the resource is associated with
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def country(self):
        """
        :returns: The ISO-3166-1 country code of the country or `all`.
        :rtype: unicode
        """
        return self._properties['country']

    @property
    def messaging_service_sid(self):
        """
        :returns: The SID of the Messaging Service used for this configuration.
        :rtype: unicode
        """
        return self._properties['messaging_service_sid']

    @property
    def date_created(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    def update(self, messaging_service_sid):
        """
        Update the MessagingConfigurationInstance

        :param unicode messaging_service_sid: The SID of the Messaging Service used for this configuration.

        :returns: The updated MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        return self._proxy.update(messaging_service_sid, )

    def fetch(self):
        """
        Fetch the MessagingConfigurationInstance

        :returns: The fetched MessagingConfigurationInstance
        :rtype: twilio.rest.verify.v2.service.messaging_configuration.MessagingConfigurationInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the MessagingConfigurationInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.MessagingConfigurationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.supersim.v1.command import CommandList
from twilio.rest.supersim.v1.fleet import FleetList
from twilio.rest.supersim.v1.ip_command import IpCommandList
from twilio.rest.supersim.v1.network import NetworkList
from twilio.rest.supersim.v1.network_access_profile import NetworkAccessProfileList
from twilio.rest.supersim.v1.sim import SimList
from twilio.rest.supersim.v1.sms_command import SmsCommandList
from twilio.rest.supersim.v1.usage_record import UsageRecordList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Supersim

        :returns: V1 version of Supersim
        :rtype: twilio.rest.supersim.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._commands = None
        self._fleets = None
        self._ip_commands = None
        self._networks = None
        self._network_access_profiles = None
        self._sims = None
        self._sms_commands = None
        self._usage_records = None

    @property
    def commands(self):
        """
        :rtype: twilio.rest.supersim.v1.command.CommandList
        """
        if self._commands is None:
            self._commands = CommandList(self)
        return self._commands

    @property
    def fleets(self):
        """
        :rtype: twilio.rest.supersim.v1.fleet.FleetList
        """
        if self._fleets is None:
            self._fleets = FleetList(self)
        return self._fleets

    @property
    def ip_commands(self):
        """
        :rtype: twilio.rest.supersim.v1.ip_command.IpCommandList
        """
        if self._ip_commands is None:
            self._ip_commands = IpCommandList(self)
        return self._ip_commands

    @property
    def networks(self):
        """
        :rtype: twilio.rest.supersim.v1.network.NetworkList
        """
        if self._networks is None:
            self._networks = NetworkList(self)
        return self._networks

    @property
    def network_access_profiles(self):
        """
        :rtype: twilio.rest.supersim.v1.network_access_profile.NetworkAccessProfileList
        """
        if self._network_access_profiles is None:
            self._network_access_profiles = NetworkAccessProfileList(self)
        return self._network_access_profiles

    @property
    def sims(self):
        """
        :rtype: twilio.rest.supersim.v1.sim.SimList
        """
        if self._sims is None:
            self._sims = SimList(self)
        return self._sims

    @property
    def sms_commands(self):
        """
        :rtype: twilio.rest.supersim.v1.sms_command.SmsCommandList
        """
        if self._sms_commands is None:
            self._sms_commands = SmsCommandList(self)
        return self._sms_commands

    @property
    def usage_records(self):
        """
        :rtype: twilio.rest.supersim.v1.usage_record.UsageRecordList
        """
        if self._usage_records is None:
            self._usage_records = UsageRecordList(self)
        return self._usage_records

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.api.v2010.account.sip.domain.auth_types import AuthTypesList
from twilio.rest.api.v2010.account.sip.domain.credential_list_mapping import CredentialListMappingList
from twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping import IpAccessControlListMappingList


class DomainList(ListResource):

    def __init__(self, version, account_sid):
        """
        Initialize the DomainList

        :param Version version: Version that contains the resource
        :param account_sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainList
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainList
        """
        super(DomainList, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, }
        self._uri = '/Accounts/{account_sid}/SIP/Domains.json'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams DomainInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists DomainInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of DomainInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return DomainPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of DomainInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return DomainPage(self._version, response, self._solution)

    def create(self, domain_name, friendly_name=values.unset,
               voice_url=values.unset, voice_method=values.unset,
               voice_fallback_url=values.unset, voice_fallback_method=values.unset,
               voice_status_callback_url=values.unset,
               voice_status_callback_method=values.unset,
               sip_registration=values.unset,
               emergency_calling_enabled=values.unset, secure=values.unset,
               byoc_trunk_sid=values.unset, emergency_caller_sid=values.unset):
        """
        Create the DomainInstance

        :param unicode domain_name: The unique address on Twilio to route SIP traffic
        :param unicode friendly_name: A string to describe the resource
        :param unicode voice_url: The URL we should call when receiving a call
        :param unicode voice_method: The HTTP method to use with voice_url
        :param unicode voice_fallback_url: The URL we should call when an error occurs in executing TwiML
        :param unicode voice_fallback_method: The HTTP method to use with voice_fallback_url
        :param unicode voice_status_callback_url: The URL that we should call to pass status updates
        :param unicode voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`
        :param bool sip_registration: Whether SIP registration is allowed
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain.
        :param bool secure: Whether secure SIP is enabled for the domain
        :param unicode byoc_trunk_sid: The SID of the BYOC Trunk resource.
        :param unicode emergency_caller_sid: Whether an emergency caller sid is configured for the domain.

        :returns: The created DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of({
            'DomainName': domain_name,
            'FriendlyName': friendly_name,
            'VoiceUrl': voice_url,
            'VoiceMethod': voice_method,
            'VoiceFallbackUrl': voice_fallback_url,
            'VoiceFallbackMethod': voice_fallback_method,
            'VoiceStatusCallbackUrl': voice_status_callback_url,
            'VoiceStatusCallbackMethod': voice_status_callback_method,
            'SipRegistration': sip_registration,
            'EmergencyCallingEnabled': emergency_calling_enabled,
            'Secure': secure,
            'ByocTrunkSid': byoc_trunk_sid,
            'EmergencyCallerSid': emergency_caller_sid,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return DomainInstance(self._version, payload, account_sid=self._solution['account_sid'], )

    def get(self, sid):
        """
        Constructs a DomainContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        return DomainContext(self._version, account_sid=self._solution['account_sid'], sid=sid, )

    def __call__(self, sid):
        """
        Constructs a DomainContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        return DomainContext(self._version, account_sid=self._solution['account_sid'], sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.DomainList>'


class DomainPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the DomainPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainPage
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        super(DomainPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DomainInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return DomainInstance(self._version, payload, account_sid=self._solution['account_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.DomainPage>'


class DomainContext(InstanceContext):

    def __init__(self, version, account_sid, sid):
        """
        Initialize the DomainContext

        :param Version version: Version that contains the resource
        :param account_sid: The SID of the Account that created the resource to fetch
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        super(DomainContext, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'sid': sid, }
        self._uri = '/Accounts/{account_sid}/SIP/Domains/{sid}.json'.format(**self._solution)

        # Dependents
        self._ip_access_control_list_mappings = None
        self._credential_list_mappings = None
        self._auth = None

    def fetch(self):
        """
        Fetch the DomainInstance

        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            sid=self._solution['sid'],
        )

    def update(self, friendly_name=values.unset, voice_fallback_method=values.unset,
               voice_fallback_url=values.unset, voice_method=values.unset,
               voice_status_callback_method=values.unset,
               voice_status_callback_url=values.unset, voice_url=values.unset,
               sip_registration=values.unset, domain_name=values.unset,
               emergency_calling_enabled=values.unset, secure=values.unset,
               byoc_trunk_sid=values.unset, emergency_caller_sid=values.unset):
        """
        Update the DomainInstance

        :param unicode friendly_name: A string to describe the resource
        :param unicode voice_fallback_method: The HTTP method used with voice_fallback_url
        :param unicode voice_fallback_url: The URL we should call when an error occurs in executing TwiML
        :param unicode voice_method: The HTTP method we should use with voice_url
        :param unicode voice_status_callback_method: The HTTP method we should use to call voice_status_callback_url
        :param unicode voice_status_callback_url: The URL that we should call to pass status updates
        :param unicode voice_url: The URL we should call when receiving a call
        :param bool sip_registration: Whether SIP registration is allowed
        :param unicode domain_name: The unique address on Twilio to route SIP traffic
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain.
        :param bool secure: Whether secure SIP is enabled for the domain
        :param unicode byoc_trunk_sid: The SID of the BYOC Trunk resource.
        :param unicode emergency_caller_sid: Whether an emergency caller sid is configured for the domain.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'VoiceFallbackMethod': voice_fallback_method,
            'VoiceFallbackUrl': voice_fallback_url,
            'VoiceMethod': voice_method,
            'VoiceStatusCallbackMethod': voice_status_callback_method,
            'VoiceStatusCallbackUrl': voice_status_callback_url,
            'VoiceUrl': voice_url,
            'SipRegistration': sip_registration,
            'DomainName': domain_name,
            'EmergencyCallingEnabled': emergency_calling_enabled,
            'Secure': secure,
            'ByocTrunkSid': byoc_trunk_sid,
            'EmergencyCallerSid': emergency_caller_sid,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the DomainInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    @property
    def ip_access_control_list_mappings(self):
        """
        Access the ip_access_control_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping.IpAccessControlListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping.IpAccessControlListMappingList
        """
        if self._ip_access_control_list_mappings is None:
            self._ip_access_control_list_mappings = IpAccessControlListMappingList(
                self._version,
                account_sid=self._solution['account_sid'],
                domain_sid=self._solution['sid'],
            )
        return self._ip_access_control_list_mappings

    @property
    def credential_list_mappings(self):
        """
        Access the credential_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.credential_list_mapping.CredentialListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.credential_list_mapping.CredentialListMappingList
        """
        if self._credential_list_mappings is None:
            self._credential_list_mappings = CredentialListMappingList(
                self._version,
                account_sid=self._solution['account_sid'],
                domain_sid=self._solution['sid'],
            )
        return self._credential_list_mappings

    @property
    def auth(self):
        """
        Access the auth

        :returns: twilio.rest.api.v2010.account.sip.domain.auth_types.AuthTypesList
        :rtype: twilio.rest.api.v2010.account.sip.domain.auth_types.AuthTypesList
        """
        if self._auth is None:
            self._auth = AuthTypesList(
                self._version,
                account_sid=self._solution['account_sid'],
                domain_sid=self._solution['sid'],
            )
        return self._auth

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.DomainContext {}>'.format(context)


class DomainInstance(InstanceResource):

    def __init__(self, version, payload, account_sid, sid=None):
        """
        Initialize the DomainInstance

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        super(DomainInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'api_version': payload.get('api_version'),
            'auth_type': payload.get('auth_type'),
            'date_created': deserialize.rfc2822_datetime(payload.get('date_created')),
            'date_updated': deserialize.rfc2822_datetime(payload.get('date_updated')),
            'domain_name': payload.get('domain_name'),
            'friendly_name': payload.get('friendly_name'),
            'sid': payload.get('sid'),
            'uri': payload.get('uri'),
            'voice_fallback_method': payload.get('voice_fallback_method'),
            'voice_fallback_url': payload.get('voice_fallback_url'),
            'voice_method': payload.get('voice_method'),
            'voice_status_callback_method': payload.get('voice_status_callback_method'),
            'voice_status_callback_url': payload.get('voice_status_callback_url'),
            'voice_url': payload.get('voice_url'),
            'subresource_uris': payload.get('subresource_uris'),
            'sip_registration': payload.get('sip_registration'),
            'emergency_calling_enabled': payload.get('emergency_calling_enabled'),
            'secure': payload.get('secure'),
            'byoc_trunk_sid': payload.get('byoc_trunk_sid'),
            'emergency_caller_sid': payload.get('emergency_caller_sid'),
        }

        # Context
        self._context = None
        self._solution = {'account_sid': account_sid, 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: DomainContext for this DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        if self._context is None:
            self._context = DomainContext(
                self._version,
                account_sid=self._solution['account_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def api_version(self):
        """
        :returns: The API version used to process the call
        :rtype: unicode
        """
        return self._properties['api_version']

    @property
    def auth_type(self):
        """
        :returns: The types of authentication mapped to the domain
        :rtype: unicode
        """
        return self._properties['auth_type']

    @property
    def date_created(self):
        """
        :returns: The RFC 2822 date and time in GMT that the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The RFC 2822 date and time in GMT that the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def domain_name(self):
        """
        :returns: The unique address on Twilio to route SIP traffic
        :rtype: unicode
        """
        return self._properties['domain_name']

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the resource
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def uri(self):
        """
        :returns: The URI of the resource, relative to `https://api.twilio.com`
        :rtype: unicode
        """
        return self._properties['uri']

    @property
    def voice_fallback_method(self):
        """
        :returns: The HTTP method used with voice_fallback_url
        :rtype: unicode
        """
        return self._properties['voice_fallback_method']

    @property
    def voice_fallback_url(self):
        """
        :returns: The URL we call when an error occurs while executing TwiML
        :rtype: unicode
        """
        return self._properties['voice_fallback_url']

    @property
    def voice_method(self):
        """
        :returns: The HTTP method to use with voice_url
        :rtype: unicode
        """
        return self._properties['voice_method']

    @property
    def voice_status_callback_method(self):
        """
        :returns: The HTTP method we use to call voice_status_callback_url
        :rtype: unicode
        """
        return self._properties['voice_status_callback_method']

    @property
    def voice_status_callback_url(self):
        """
        :returns: The URL that we call with status updates
        :rtype: unicode
        """
        return self._properties['voice_status_callback_url']

    @property
    def voice_url(self):
        """
        :returns: The URL we call when receiving a call
        :rtype: unicode
        """
        return self._properties['voice_url']

    @property
    def subresource_uris(self):
        """
        :returns: A list mapping resources associated with the SIP Domain resource
        :rtype: unicode
        """
        return self._properties['subresource_uris']

    @property
    def sip_registration(self):
        """
        :returns: Whether SIP registration is allowed
        :rtype: bool
        """
        return self._properties['sip_registration']

    @property
    def emergency_calling_enabled(self):
        """
        :returns: Whether emergency calling is enabled for the domain.
        :rtype: bool
        """
        return self._properties['emergency_calling_enabled']

    @property
    def secure(self):
        """
        :returns: Whether secure SIP is enabled for the domain
        :rtype: bool
        """
        return self._properties['secure']

    @property
    def byoc_trunk_sid(self):
        """
        :returns: The SID of the BYOC Trunk resource.
        :rtype: unicode
        """
        return self._properties['byoc_trunk_sid']

    @property
    def emergency_caller_sid(self):
        """
        :returns: Whether an emergency caller sid is configured for the domain.
        :rtype: unicode
        """
        return self._properties['emergency_caller_sid']

    def fetch(self):
        """
        Fetch the DomainInstance

        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return self._proxy.fetch()

    def update(self, friendly_name=values.unset, voice_fallback_method=values.unset,
               voice_fallback_url=values.unset, voice_method=values.unset,
               voice_status_callback_method=values.unset,
               voice_status_callback_url=values.unset, voice_url=values.unset,
               sip_registration=values.unset, domain_name=values.unset,
               emergency_calling_enabled=values.unset, secure=values.unset,
               byoc_trunk_sid=values.unset, emergency_caller_sid=values.unset):
        """
        Update the DomainInstance

        :param unicode friendly_name: A string to describe the resource
        :param unicode voice_fallback_method: The HTTP method used with voice_fallback_url
        :param unicode voice_fallback_url: The URL we should call when an error occurs in executing TwiML
        :param unicode voice_method: The HTTP method we should use with voice_url
        :param unicode voice_status_callback_method: The HTTP method we should use to call voice_status_callback_url
        :param unicode voice_status_callback_url: The URL that we should call to pass status updates
        :param unicode voice_url: The URL we should call when receiving a call
        :param bool sip_registration: Whether SIP registration is allowed
        :param unicode domain_name: The unique address on Twilio to route SIP traffic
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain.
        :param bool secure: Whether secure SIP is enabled for the domain
        :param unicode byoc_trunk_sid: The SID of the BYOC Trunk resource.
        :param unicode emergency_caller_sid: Whether an emergency caller sid is configured for the domain.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            voice_fallback_method=voice_fallback_method,
            voice_fallback_url=voice_fallback_url,
            voice_method=voice_method,
            voice_status_callback_method=voice_status_callback_method,
            voice_status_callback_url=voice_status_callback_url,
            voice_url=voice_url,
            sip_registration=sip_registration,
            domain_name=domain_name,
            emergency_calling_enabled=emergency_calling_enabled,
            secure=secure,
            byoc_trunk_sid=byoc_trunk_sid,
            emergency_caller_sid=emergency_caller_sid,
        )

    def delete(self):
        """
        Deletes the DomainInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    @property
    def ip_access_control_list_mappings(self):
        """
        Access the ip_access_control_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping.IpAccessControlListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping.IpAccessControlListMappingList
        """
        return self._proxy.ip_access_control_list_mappings

    @property
    def credential_list_mappings(self):
        """
        Access the credential_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.credential_list_mapping.CredentialListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.credential_list_mapping.CredentialListMappingList
        """
        return self._proxy.credential_list_mappings

    @property
    def auth(self):
        """
        Access the auth

        :returns: twilio.rest.api.v2010.account.sip.domain.auth_types.AuthTypesList
        :rtype: twilio.rest.api.v2010.account.sip.domain.auth_types.AuthTypesList
        """
        return self._proxy.auth

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.DomainInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.api.v2010 import V2010


class Api(Domain):

    def __init__(self, twilio):
        """
        Initialize the Api Domain

        :returns: Domain for Api
        :rtype: twilio.rest.api.Api
        """
        super(Api, self).__init__(twilio)

        self.base_url = 'https://api.twilio.com'

        # Versions
        self._v2010 = None

    @property
    def v2010(self):
        """
        :returns: Version v2010 of api
        :rtype: twilio.rest.api.v2010.V2010
        """
        if self._v2010 is None:
            self._v2010 = V2010(self)
        return self._v2010

    @property
    def account(self):
        """
        :returns: Account provided as the authenticating account
        :rtype: twilio.rest.api.v2010.account.AccountContext
        """
        return self.v2010.account

    @property
    def accounts(self):
        """
        :rtype: twilio.rest.api.v2010.account.AccountList
        """
        return self.v2010.accounts

    @property
    def addresses(self):
        """
        :rtype: twilio.rest.api.v2010.account.address.AddressList
        """
        return self.account.addresses

    @property
    def applications(self):
        """
        :rtype: twilio.rest.api.v2010.account.application.ApplicationList
        """
        return self.account.applications

    @property
    def authorized_connect_apps(self):
        """
        :rtype: twilio.rest.api.v2010.account.authorized_connect_app.AuthorizedConnectAppList
        """
        return self.account.authorized_connect_apps

    @property
    def available_phone_numbers(self):
        """
        :rtype: twilio.rest.api.v2010.account.available_phone_number.AvailablePhoneNumberCountryList
        """
        return self.account.available_phone_numbers

    @property
    def balance(self):
        """
        :rtype: twilio.rest.api.v2010.account.balance.BalanceList
        """
        return self.account.balance

    @property
    def calls(self):
        """
        :rtype: twilio.rest.api.v2010.account.call.CallList
        """
        return self.account.calls

    @property
    def conferences(self):
        """
        :rtype: twilio.rest.api.v2010.account.conference.ConferenceList
        """
        return self.account.conferences

    @property
    def connect_apps(self):
        """
        :rtype: twilio.rest.api.v2010.account.connect_app.ConnectAppList
        """
        return self.account.connect_apps

    @property
    def incoming_phone_numbers(self):
        """
        :rtype: twilio.rest.api.v2010.account.incoming_phone_number.IncomingPhoneNumberList
        """
        return self.account.incoming_phone_numbers

    @property
    def keys(self):
        """
        :rtype: twilio.rest.api.v2010.account.key.KeyList
        """
        return self.account.keys

    @property
    def messages(self):
        """
        :rtype: twilio.rest.api.v2010.account.message.MessageList
        """
        return self.account.messages

    @property
    def new_keys(self):
        """
        :rtype: twilio.rest.api.v2010.account.new_key.NewKeyList
        """
        return self.account.new_keys

    @property
    def new_signing_keys(self):
        """
        :rtype: twilio.rest.api.v2010.account.new_signing_key.NewSigningKeyList
        """
        return self.account.new_signing_keys

    @property
    def notifications(self):
        """
        :rtype: twilio.rest.api.v2010.account.notification.NotificationList
        """
        return self.account.notifications

    @property
    def outgoing_caller_ids(self):
        """
        :rtype: twilio.rest.api.v2010.account.outgoing_caller_id.OutgoingCallerIdList
        """
        return self.account.outgoing_caller_ids

    @property
    def queues(self):
        """
        :rtype: twilio.rest.api.v2010.account.queue.QueueList
        """
        return self.account.queues

    @property
    def recordings(self):
        """
        :rtype: twilio.rest.api.v2010.account.recording.RecordingList
        """
        return self.account.recordings

    @property
    def signing_keys(self):
        """
        :rtype: twilio.rest.api.v2010.account.signing_key.SigningKeyList
        """
        return self.account.signing_keys

    @property
    def sip(self):
        """
        :rtype: twilio.rest.api.v2010.account.sip.SipList
        """
        return self.account.sip

    @property
    def short_codes(self):
        """
        :rtype: twilio.rest.api.v2010.account.short_code.ShortCodeList
        """
        return self.account.short_codes

    @property
    def tokens(self):
        """
        :rtype: twilio.rest.api.v2010.account.token.TokenList
        """
        return self.account.tokens

    @property
    def transcriptions(self):
        """
        :rtype: twilio.rest.api.v2010.account.transcription.TranscriptionList
        """
        return self.account.transcriptions

    @property
    def usage(self):
        """
        :rtype: twilio.rest.api.v2010.account.usage.UsageList
        """
        return self.account.usage

    @property
    def validation_requests(self):
        """
        :rtype: twilio.rest.api.v2010.account.validation_request.ValidationRequestList
        """
        return self.account.validation_requests

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class VerificationAttemptList(ListResource):

    def __init__(self, version):
        """
        Initialize the VerificationAttemptList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptList
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptList
        """
        super(VerificationAttemptList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Attempts'.format(**self._solution)

    def stream(self, date_created_after=values.unset,
               date_created_before=values.unset, channel_data_to=values.unset,
               limit=None, page_size=None):
        """
        Streams VerificationAttemptInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param datetime date_created_after: Filter verification attempts after this date
        :param datetime date_created_before: Filter verification attempts befor this date
        :param unicode channel_data_to: Destination of a verification
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            date_created_after=date_created_after,
            date_created_before=date_created_before,
            channel_data_to=channel_data_to,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, date_created_after=values.unset,
             date_created_before=values.unset, channel_data_to=values.unset,
             limit=None, page_size=None):
        """
        Lists VerificationAttemptInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param datetime date_created_after: Filter verification attempts after this date
        :param datetime date_created_before: Filter verification attempts befor this date
        :param unicode channel_data_to: Destination of a verification
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance]
        """
        return list(self.stream(
            date_created_after=date_created_after,
            date_created_before=date_created_before,
            channel_data_to=channel_data_to,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, date_created_after=values.unset,
             date_created_before=values.unset, channel_data_to=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of VerificationAttemptInstance records from the API.
        Request is executed immediately

        :param datetime date_created_after: Filter verification attempts after this date
        :param datetime date_created_before: Filter verification attempts befor this date
        :param unicode channel_data_to: Destination of a verification
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptPage
        """
        data = values.of({
            'DateCreatedAfter': serialize.iso8601_datetime(date_created_after),
            'DateCreatedBefore': serialize.iso8601_datetime(date_created_before),
            'ChannelData.To': channel_data_to,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return VerificationAttemptPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of VerificationAttemptInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return VerificationAttemptPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a VerificationAttemptContext

        :param sid: Verification Attempt Sid.

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        """
        return VerificationAttemptContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a VerificationAttemptContext

        :param sid: Verification Attempt Sid.

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        """
        return VerificationAttemptContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.VerificationAttemptList>'


class VerificationAttemptPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the VerificationAttemptPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptPage
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptPage
        """
        super(VerificationAttemptPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of VerificationAttemptInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        """
        return VerificationAttemptInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.VerificationAttemptPage>'


class VerificationAttemptContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the VerificationAttemptContext

        :param Version version: Version that contains the resource
        :param sid: Verification Attempt Sid.

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        """
        super(VerificationAttemptContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Attempts/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the VerificationAttemptInstance

        :returns: The fetched VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return VerificationAttemptInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.VerificationAttemptContext {}>'.format(context)


class VerificationAttemptInstance(InstanceResource):

    class Channels(object):
        SMS = "sms"
        CALL = "call"
        EMAIL = "email"
        WHATSAPP = "whatsapp"

    class ConversionStatus(object):
        CONVERTED = "converted"
        UNCONVERTED = "unconverted"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the VerificationAttemptInstance

        :returns: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        """
        super(VerificationAttemptInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'conversion_status': payload.get('conversion_status'),
            'channel': payload.get('channel'),
            'channel_data': payload.get('channel_data'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: VerificationAttemptContext for this VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptContext
        """
        if self._context is None:
            self._context = VerificationAttemptContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies this Verification Attempt
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: Account Sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The service_sid
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def date_created(self):
        """
        :returns: The date this Attempt was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date this Attempt was updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def conversion_status(self):
        """
        :returns: Status of a conversion
        :rtype: VerificationAttemptInstance.ConversionStatus
        """
        return self._properties['conversion_status']

    @property
    def channel(self):
        """
        :returns: Channel used for the attempt
        :rtype: VerificationAttemptInstance.Channels
        """
        return self._properties['channel']

    @property
    def channel_data(self):
        """
        :returns: Object with the channel information for an attempt
        :rtype: dict
        """
        return self._properties['channel_data']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the VerificationAttemptInstance

        :returns: The fetched VerificationAttemptInstance
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.VerificationAttemptInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class PlaybackGrantList(ListResource):

    def __init__(self, version, sid):
        """
        Initialize the PlaybackGrantList

        :param Version version: Version that contains the resource
        :param sid: The unique string that identifies the PlayerStreamer associated with this PlaybackGrant.

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantList
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantList
        """
        super(PlaybackGrantList, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }

    def get(self):
        """
        Constructs a PlaybackGrantContext

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        """
        return PlaybackGrantContext(self._version, sid=self._solution['sid'], )

    def __call__(self):
        """
        Constructs a PlaybackGrantContext

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        """
        return PlaybackGrantContext(self._version, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Media.V1.PlaybackGrantList>'


class PlaybackGrantPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the PlaybackGrantPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param sid: The unique string that identifies the PlayerStreamer associated with this PlaybackGrant.

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantPage
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantPage
        """
        super(PlaybackGrantPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of PlaybackGrantInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        return PlaybackGrantInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Media.V1.PlaybackGrantPage>'


class PlaybackGrantContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the PlaybackGrantContext

        :param Version version: Version that contains the resource
        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        """
        super(PlaybackGrantContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/PlayerStreamers/{sid}/PlaybackGrant'.format(**self._solution)

    def create(self, ttl=values.unset, access_control_allow_origin=values.unset):
        """
        Create the PlaybackGrantInstance

        :param unicode ttl: The time to live of the PlaybackGrant
        :param unicode access_control_allow_origin: The full URL that is authorized to play back the livestream

        :returns: The created PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        data = values.of({'Ttl': ttl, 'AccessControlAllowOrigin': access_control_allow_origin, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return PlaybackGrantInstance(self._version, payload, sid=self._solution['sid'], )

    def fetch(self):
        """
        Fetch the PlaybackGrantInstance

        :returns: The fetched PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return PlaybackGrantInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Media.V1.PlaybackGrantContext {}>'.format(context)


class PlaybackGrantInstance(InstanceResource):

    def __init__(self, version, payload, sid):
        """
        Initialize the PlaybackGrantInstance

        :returns: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        super(PlaybackGrantInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'url': payload.get('url'),
            'account_sid': payload.get('account_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'grant': payload.get('grant'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: PlaybackGrantContext for this PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantContext
        """
        if self._context is None:
            self._context = PlaybackGrantContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the PlayerStreamer associated with this PlaybackGrant.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def grant(self):
        """
        :returns: The grant that authorizes the player sdk to connect to the livestream
        :rtype: dict
        """
        return self._properties['grant']

    def create(self, ttl=values.unset, access_control_allow_origin=values.unset):
        """
        Create the PlaybackGrantInstance

        :param unicode ttl: The time to live of the PlaybackGrant
        :param unicode access_control_allow_origin: The full URL that is authorized to play back the livestream

        :returns: The created PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        return self._proxy.create(ttl=ttl, access_control_allow_origin=access_control_allow_origin, )

    def fetch(self):
        """
        Fetch the PlaybackGrantInstance

        :returns: The fetched PlaybackGrantInstance
        :rtype: twilio.rest.media.v1.player_streamer.playback_grant.PlaybackGrantInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Media.V1.PlaybackGrantInstance {}>'.format(context)
